#Dataset simulation
library(MASS)
library(MatchIt)
library(survival)

BETWEEN=function(x,ul){
  l=ul[1]
  u=ul[2]
  return(x>=l & x<=u)
}
BETWEENAll=function(xul){
  x=xul[1]
  l=xul[2]
  u=xul[3]
  return(x>=l & x<=u)
}
assignbeta=function(count,beta){
  j=1
  for (i in 1:length(count)) {
    if (count[i]==1) {
      count[i]=beta[j]
      j=j+1
    }
  }
  return(count)
}


#covariate effect 
Beta=as.matrix(c(c(-6,0.1),c(0.15,-0.2,-0.5,0.25,-0.2,0.5,0.25,0,0,-0.2,-0.2,0,0,0,0)))
Beta_intr=as.matrix(c(-0.75,0.5,0.25,-1.25,1,0,0,0,0,0,0,0,0,0,0))
#n: number of subjects, p: number of covariates
p=15
n=300
#Decrease the scale to decrease the level of noise the random error
scale=1/2
#Function to simulate survival time based on the AFT model (random error follows an extreme value distribution)
simuAFT=function(seed,n,scale){
  set.seed(seed)
  Trt=c(rep(0.5,n/2),rep(-0.5,n/2))
  Sigma <- matrix(c(1,0.5,0.5,1),2,2)
  Z1=rnorm(n,0,1)
  Z2=rnorm(n,0,1)
  Z3=rnorm(n,0,1)
  Z4=ifelse(runif(n,0,1)>0.5,0.5,-0.5)
  Z5=ifelse(runif(n,0,1)>0.5,0.5,-0.5)
  Z6=rnorm(n,0,1)
  Z7=rnorm(n,0,1)
  Z8=rnorm(n,0,1)
  Z9=rnorm(n,0,1)
  Zcor1=mvrnorm(n,rep(0,2), Sigma)
  Z10=Zcor1[,1]
  Z11=Zcor1[,2]
  Z12=rnorm(n,0,1)
  Z13=rnorm(n,0,1)
  Z14=ifelse(runif(n,0,1)>0.5,0.5,-0.5)
  Z15=ifelse(runif(n,0,1)>0.5,0.5,-0.5)
  Z=cbind(Int=rep(1,n),Trt,Z1,Z2,Z3,Z4,Z5,Z6,Z7,Z8,Z9,Z10,Z11,Z12,Z13,Z14,Z15)
  Z_int=cbind(Trt*Z1,Trt*Z2,Trt*Z3,Trt*Z4,Trt*Z5,Trt*Z6,Trt*Z7,Trt*Z8,Trt*Z9,Trt*Z10,Trt*Z11,Trt*Z12,Trt*Z13,Trt*Z14,Trt*Z15)
  UU = runif(n)
  WW = log(-log(UU))*scale
  YY = exp(Z%*%Beta+Z_int%*%Beta_intr+WW)
  V1 = runif(n)
  V2 = runif(n)
  CC = ifelse(V2<0.5, -log(V1)/120,-log(V1)*0.125)
  time = ifelse(YY<=CC,YY,CC)
  status = ifelse(YY<=CC,1,0)
  dataf=data.frame(ID=1:n,time,status=status,groupId=ifelse(Trt==0.5,1,0),eventT=YY,Z[,-1])
  return(dataf)
}

seed=5

##############################testing sample############################################################################
##############################testing sample############################################################################
##############################testing sample############################################################################
##############################testing sample############################################################################

#TS stands for two samples
#Simulate the testing sample in the two independent samples case
seed.TS=seed+2000
mydata.TS=simuAFT(seed=seed.TS,n,scale)
nameM.TS=names(mydata.TS)[7:(6+p)]
nameid.TS=match(nameM.TS,colnames(mydata.TS))
#Create matched pairs using r package "MatchIt"
mydata1.TS=mydata.TS
name.matching.TS=names(mydata1.TS[,-c(1:6)])
f.matching.TS <- as.formula(paste("groupId~", paste(name.matching.TS, collapse = " + ")))
#Create matched pairs using mahalanobis distance
m.mahalp.TS	<- matchit(f.matching.TS,data=mydata1.TS, method = "nearest",distance = "mahalanobis",replace = FALSE)
#Create matched pairs using propensity score
#m.mahalp.TS	<- matchit(f.matching.TS, data=mydata1.TS, method = "nearest",distance = "glm", replace = FALSE)}

pair.TS0=cbind(rownames(m.mahalp.TS[[1]]),m.mahalp.TS[[1]])
pair.TS0=pair.TS0[which(!is.na(pair.TS0[,2])),]
CBT0.TS = mydata1.TS[pair.TS0[,1],]
SBT0.TS = mydata1.TS[pair.TS0[,2],]
dLogTime.TS=log(CBT0.TS[,"eventT"])-log(SBT0.TS[,"eventT"])
DZ.TS=CBT0.TS[,nameid.TS]-SBT0.TS[,nameid.TS]
AZ.TS=(CBT0.TS[,nameid.TS]+SBT0.TS[,nameid.TS])/2
colnames(DZ.TS)=paste0("d_",nameM.TS)
colnames(AZ.TS)=paste0("a_",nameM.TS)

KMCen.TS=survfit(Surv(time,1-status)~1,data=mydata.TS)
CBT0pred.TS=cbind(summary(KMCen.TS, times =CBT0.TS[,"time"])[[2]],summary(KMCen.TS, times =CBT0.TS[,"time"])[[6]])
CBT0pred.TS=CBT0pred.TS[match(CBT0.TS[,"time"],CBT0pred.TS[,1]),]
SBT0pred.TS=cbind(summary(KMCen.TS, times =SBT0.TS[,"time"])[[2]],summary(KMCen.TS, times =SBT0.TS[,"time"])[[6]])
SBT0pred.TS=SBT0pred.TS[match(SBT0.TS[,"time"],SBT0pred.TS[,1]),]
comp.TS = CBT0.TS[,"status"]*SBT0.TS[,"status"]
bivS.TS = CBT0pred.TS[,2]*SBT0pred.TS[,2]
wgt.TS = comp.TS/bivS.TS

pairdata.TS=data.frame(dLogTime.TS,DZ.TS,AZ.TS)
npair.TS=dim(pairdata.TS)[1]
Z.TS=pairdata.TS[,2:(1+2*p)]


##############################training sample############################################################################
##############################training sample############################################################################
##############################training sample############################################################################
##############################training sample############################################################################
#Simulate the training sample in the two independent samples case
mydata=simuAFT(seed=seed,n,scale)[,-5]
nameM=names(mydata)[6:(5+p)]
nameid=match(nameM,colnames(mydata))
#Matching
mydata1=mydata[which(mydata[,"status"]==1),]
np=dim(mydata1)[1]
rownames(mydata1)=c(1:np)
name.matching=names(mydata1[,-c(1:5)])
f.matching <- as.formula(paste("groupId~", paste(name.matching, collapse = " + ")))
#Create matched pairs using mahalanobis distance
m.mahalp	<- matchit(f.matching.TS,data=mydata1, method = "nearest",distance = "mahalanobis",replace = FALSE)
#Create matched pairs using propensity score
#m.mahalp	<- matchit(f.matching.TS, data=mydata1, method = "nearest",distance = "glm", replace = FALSE)}

pair=cbind(rownames(m.mahalp[[1]]),m.mahalp[[1]])
pair=pair[which(!is.na(pair[,2])),]
CBT0 = mydata1[pair[,1],]
SBT0 = mydata1[pair[,2],]
dLogTime=log(CBT0[,2])-log(SBT0[,2])
DZ=CBT0[,nameid]-SBT0[,nameid]
AZ=(CBT0[,nameid]+SBT0[,nameid])/2
colnames(DZ)=paste0("d_",nameM)
colnames(AZ)=paste0("a_",nameM)
#Calculate the IPCW by matched pairs, we use KM estimator to estimate the distribution of the censoring time
KMCen=survfit(Surv(time,1-status)~1,data=mydata)
CBT0pred=cbind(summary(KMCen, times =CBT0[,"time"])[[2]],summary(KMCen, times =CBT0[,"time"])[[6]])
CBT0pred=CBT0pred[match(CBT0[,"time"],CBT0pred[,1]),]
SBT0pred=cbind(summary(KMCen, times =SBT0[,"time"])[[2]],summary(KMCen, times =SBT0[,"time"])[[6]])
SBT0pred=SBT0pred[match(SBT0[,"time"],SBT0pred[,1]),]
comp = CBT0[,"status"]*SBT0[,"status"]
bivS = CBT0pred[,2]*SBT0pred[,2]
wgt = comp/bivS
#Pairdata is the paired modified dataset for estimating the causal effect modifier
pairdata=data.frame(dLogTime,DZ,AZ)
npair=dim(pairdata)[1]
rownames(pairdata)=c(1:npair)
#create bootstrapping sample ID for calculating the out-of-bag (OOB) metric
nboot=1000
bootsamid1=matrix(sample(1:npair,size=npair*nboot,replace=TRUE),nrow=nboot)
